import gi

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
gi.require_version('Notify', '0.7')

import os
import sys

import humanize
from gi.repository import GLib, GObject, Gio, Gtk, Notify

from abeluna.sync import server
from abeluna.util import colour_text
from abeluna.windows import SettingsWindow, TodoListWindow


UI_LOCATION = os.path.join(os.path.dirname(__file__), 'ui')


class MainWindow(Gtk.ApplicationWindow):
    def __init__(self, app):
        Gtk.Window.__init__(self, title='Abeluna', application=app)
        self.set_default_size(1200, 600)

        self.todolist_window = TodoListWindow()

        self.connect('button-press-event', self.todolist_window.reset_action_popover)
        self.connect('configure-event', self.todolist_window.reset_action_popover)

        self.new_todo_action = Gio.SimpleAction.new('new-todo', None)
        self.new_todo_action.connect('activate', lambda action, parameter: self.todolist_window.new_todo())
        self.add_action(self.new_todo_action)
        self.sync_todo_action = Gio.SimpleAction.new('sync-todo', None)
        self.sync_todo_action.connect('activate', lambda action, parameter: server.synchronize_todolist())
        self.add_action(self.sync_todo_action)
        self.general_settings_action = Gio.SimpleAction.new('general-settings', None)
        self.general_settings_action.connect(
            'activate',
            lambda action, parameter: SettingsWindow(parent=self, active_child='General'),
        )
        self.add_action(self.general_settings_action)
        self.calendar_settings_action = Gio.SimpleAction.new('calendar-settings', None)
        self.calendar_settings_action.connect(
            'activate',
            lambda action, parameter: SettingsWindow(parent=self, active_child='Calendars'),
        )
        self.add_action(self.calendar_settings_action)

        self.main_grid = Gtk.Grid()
        self.main_grid.set_column_homogeneous(True)
        self.main_grid.set_column_spacing(5)
        self.main_grid.set_row_homogeneous(True)
        self.main_grid.set_row_spacing(5)
        self.main_grid.set_border_width(10)
        self.add(self.main_grid)

        self.calendar_store = Gtk.ListStore(str, str)
        self.calendar_uid_to_iter = {}
        self.calendar_tree_view = Gtk.TreeView(model=self.calendar_store)
        self.calendar_tree_view.append_column(Gtk.TreeViewColumn('Calendars', Gtk.CellRendererText(), text=0))
        self.calendar_tree_view.get_selection().connect('changed', self.calendar_tree_selection_changed)
        self.calendar_tree_view.set_enable_search(False)
        self.calendar_tree_view.set_search_column(-1)
        self.calendar_tree_view.set_tooltip_column(0)

        self.calendar_scrollable_view = Gtk.ScrolledWindow(vexpand=True)
        self.calendar_scrollable_view.set_propagate_natural_width(True)
        self.calendar_scrollable_view.set_shadow_type(type=Gtk.ShadowType.ETCHED_OUT)
        self.calendar_scrollable_view.add(self.calendar_tree_view)

        self.main_grid.attach(self.calendar_scrollable_view, 0, 0, 4, 19)

        self.status_view = Gtk.Grid()
        self.status_view.set_column_spacing(5)
        self.status_view.set_hexpand(True)
        self.status_view.set_border_width(5)
        self.sync_spinner = Gtk.Spinner()
        self.sync_spinner.set_halign(Gtk.Align.START)
        self.sync_spinner.set_hexpand(False)
        self.status_view.add(self.sync_spinner)
        self.sync_label = Gtk.Label(label=' ')
        self.sync_label.set_halign(Gtk.Align.END)
        self.sync_label.set_hexpand(True)
        self.sync_label.set_property('use-markup', True)
        self.status_view.add(self.sync_label)
        self.main_grid.attach_next_to(
            self.status_view, self.calendar_scrollable_view, Gtk.PositionType.BOTTOM, 4, 1,
        )

        self.main_grid.attach_next_to(
            self.todolist_window, self.calendar_scrollable_view, Gtk.PositionType.RIGHT, 13, 20,
        )

        self.main_grid.show_all()

        GObject.timeout_add_seconds(30, self.update_natural_dates)
        server.refresh_calendars()
        self.rebuild_calendarlist()
        server.sync_connect(self.on_todo_sync)

    def on_todo_sync_watcher(self):
        if self._syncing:
            self.sync_spinner.start()
            return True
        self.todolist_window.rebuild_todolist()
        self.sync_spinner.stop()
        self.update_natural_dates()
        return False

    def on_todo_sync(self, mode):
        if mode == 'PRE_SYNC':
            self._syncing = True
            GObject.timeout_add(200, self.on_todo_sync_watcher)
        elif mode == 'POST_SYNC':
            self._syncing = False

    def update_natural_dates(self):
        last_sync = server.last_sync
        if last_sync is None:
            self.sync_label.set_label(' ')
            self.sync_label.set_tooltip_text('')
        else:
            self.sync_label.set_label(colour_text('Last synced {}.'.format(humanize.naturaltime(last_sync)), '#666'))
            self.sync_label.set_tooltip_text(last_sync.strftime('%c'))
        return True

    def rebuild_calendarlist(self):
        path_iter = self.calendar_tree_view.get_selection().get_selected()[1]
        if path_iter is None:
            _currently_selected_uid = None
        else:
            _currently_selected_uid = self.calendar_store[path_iter][1]
        self.calendar_store.clear()
        self.calendar_uid_to_iter.clear()

        for uid, calendar in server.calendars.items():
            self.calendar_uid_to_iter[uid] = self.calendar_store.append([calendar.name, uid])

        try:
            self.calendar_tree_view.get_selection().select_iter(
                self.calendar_uid_to_iter[_currently_selected_uid],
            )
        except KeyError:
            self.calendar_tree_selection_changed()

    def calendar_tree_selection_changed(self, calendar_tree_selection=None):
        self.todolist_window.reset_action_popover()
        if calendar_tree_selection is None:
            calendar_tree_selection = self.calendar_tree_view.get_selection()

        path_iter = calendar_tree_selection.get_selected()[1]
        if path_iter is None:
            self.new_todo_action.set_enabled(False)
            self.todolist_window.current_calendar = None
        else:
            self.new_todo_action.set_enabled(True)
            self.todolist_window.current_calendar = self.calendar_store[path_iter][1]


class Abeluna(Gtk.Application):
    def __init__(self):
        Gtk.Application.__init__(self)

    def do_activate(self):
        win = MainWindow(self)
        win.show()

    def do_startup(self):
        Gtk.Application.do_startup(self)
        builder = Gtk.Builder()
        Notify.init('Abeluna')
        try:
            builder.add_from_file(os.path.join(UI_LOCATION, 'menubar.ui'))
        except GLib.Error:
            self.quit()

        self.set_menubar(builder.get_object('menubar'))

        quit_action = Gio.SimpleAction.new('quit', None)
        quit_action.connect('activate', lambda action, parameter: self.quit())
        self.add_action(quit_action)

        self.add_accelerator('<Ctrl>N', 'win.new-todo', None)
        self.add_accelerator('<Ctrl>R', 'win.sync-todo', None)
        self.add_accelerator('<Ctrl>W', 'app.quit', None)


def main():
    app = Abeluna()
    try:
        exit_code = app.run(sys.argv)
    except KeyboardInterrupt:
        exit_code = 1
    server.stop_all()
    return exit_code


if __name__ == '__main__':
    main()



import calendar
import datetime

import pytz
from gi.repository import GObject, Gtk

from abeluna.settings import settings
from abeluna.widgets.dropdown_select import DropdownSelectWidget


class DateTimePickerWidget(Gtk.Grid):
    __gsignals__ = {
        'updated-date': (GObject.SignalFlags.RUN_LAST, None, ()),
    }

    def __init__(self, *args, **kwargs):
        self.date_only = kwargs.pop('date_only', True)
        selected_date = kwargs.pop('selected_date', None)
        self._tz = kwargs.pop('timezone', None)

        super().__init__(*args, **kwargs)
        self.set_column_spacing(10)
        self.set_row_spacing(10)
        self.set_border_width(5)

        self._create_widgets()

        self.set_selected_date(selected_date)
        self.set_date_only(self.date_only)

    @property
    def now(self):
        return datetime.datetime.now(tz=self.tz) + datetime.timedelta(minutes=1)

    @property
    def tz(self):
        if self._tz is not None:
            return self._tz
        return pytz.timezone(settings.TIMEZONE)

    def get_date_only(self):
        return self.date_only

    def set_date_only(self, date_only):
        self.date_only = date_only
        for child in self.get_children():
            self.remove(child)

        if not self.date_only:
            self.attach(self.hour_selector, 0, 0, 2, 3)
            self.attach_next_to(self.time_colon, self.hour_selector, Gtk.PositionType.RIGHT, 1, 3)
            self.attach_next_to(self.minute_selector, self.time_colon, Gtk.PositionType.RIGHT, 2, 3)
        self.attach(self.year_selector, 6, 0, 3, 1)
        self.attach_next_to(self.month_selector, self.year_selector, Gtk.PositionType.BOTTOM, 3, 1)
        self.attach_next_to(self.day_selector, self.month_selector, Gtk.PositionType.BOTTOM, 3, 1)

        self.attach(self.set_button, 7, 4, 2, 1)

        # self.emit('updated-date')

    def get_selected_date(self):
        return self.selected_date

    def set_selected_date(self, selected_date):
        self.selected_date = selected_date
        if self.selected_date is not None:
            date = self.selected_date = self.selected_date.astimezone(tz=self.tz)
        else:
            date = self.now

        self.hour_selector.set_value(date.hour)
        self.minute_selector.set_value(date.minute)
        self.year_selector.set_value(date.year)
        self.month_selector.set_active(date.month - 1)
        self.day_selector.set_value(date.day)

        # self.emit('updated-date')

    def _create_widgets(self):
        self.hour_selector = Gtk.SpinButton(
            adjustment=Gtk.Adjustment(
                value=0,
                lower=0,
                upper=23,
                step_increment=1,
                page_increment=0,
                page_size=0,
            ),
        )
        self.hour_selector.set_wrap(True)
        self.hour_selector.set_orientation(Gtk.Orientation.VERTICAL)

        self.time_colon = Gtk.Label(label=':')

        self.minute_selector = Gtk.SpinButton(
            adjustment=Gtk.Adjustment(
                value=0,
                lower=0,
                upper=59,
                step_increment=1,
                page_increment=5,
                page_size=0,
            ),
        )
        self.minute_selector.set_wrap(True)
        self.minute_selector.set_orientation(Gtk.Orientation.VERTICAL)
        self.minute_selector.set_margin_end(20)

        self.year_selector = Gtk.SpinButton(
            adjustment=Gtk.Adjustment(
                value=self.now.year,
                lower=datetime.MINYEAR,
                upper=datetime.MAXYEAR,
                step_increment=1,
                page_increment=0,
                page_size=0,
            ),
        )
        self.year_selector.set_wrap(True)

        months = []
        for month_index in range(1, 13):
            months.append([str(month_index), datetime.date(1970, month_index, 1).strftime('%B')])
        self.month_selector = DropdownSelectWidget(options=months)
        self.month_selector.set_active(0)

        day_adjustment = Gtk.Adjustment(
            value=1,
            lower=1,
            upper=31,
            step_increment=1,
            page_increment=5,
            page_size=0,
        )
        self.day_selector = Gtk.SpinButton(adjustment=day_adjustment)
        self.day_selector.set_wrap(True)
        self.day_selector.set_hexpand(True)

        def on_changed(*args, **kwargs):
            month = self.month_selector.get_active_id()
            if month is None:
                return
            month = int(month)
            year = self.year_selector.get_value_as_int()
            weekday, num_days = calendar.monthrange(year, month)
            day_adjustment.set_upper(num_days)
            self.day_selector.update()
        self.month_selector.connect('changed', on_changed)
        self.year_selector.connect('value-changed', on_changed)

        self.set_button = Gtk.Button(label='Set!')

        def on_set_button_clicked(button):
            self.selected_date = datetime.datetime(
                year=self.year_selector.get_value_as_int(),
                month=int(self.month_selector.get_active_id()),
                day=self.day_selector.get_value_as_int(),
                hour=self.hour_selector.get_value_as_int(),
                minute=self.minute_selector.get_value_as_int(),
            ).astimezone(tz=self.tz)
            self.emit('updated-date')
        self.set_button.connect('clicked', on_set_button_clicked)
        self.set_button.set_margin_top(20)
